module Splot
  class Runner
    attr :params

    def self.run!
      new.tap(&:parse).run!
    end

    def initialize
      @params = {}
    end

    def command
      ::Splot.command params
    end

    def parse
      parse_splotrc
      parse_command_line_arguments
    end

    def parse_command_line_arguments
      OptionParser.new do |opts|
        opts.banner = "Usage: #{File.basename($PROGRAM_NAME)} [options]"

        opts.on('-f', '--file FILE', 'Run test for a given FILE') do |file|
          params[:file] = file
        end

        opts.on('-i', '--include-path PATH', 'Add an include PATH') do |path|
          params[:include_path] = path
        end

        opts.on('-l', '--line LINE', 'Only run test on LINE') do |line|
          params[:line] = line.to_i
        end

        opts.on('-r', '--runner RUNNER', 'Specify alternate runner') do |runner|
          params[:runner] = runner
        end

        opts.on('-p', '--preloader PRELOADER', 'Use e.g. zeus or spring to preload the environment') do |preloader|
          params[:preloader] = preloader
        end
      end.parse!
    end

    def parse_splotrc
      params[:config_file] = splotrc if File.size? splotrc
    end

    def run!
      Kernel.exec command
    rescue RunnerNotFoundError => e
      warn "Warning: #{e}; will not run any test"
      exit 2
    end

    private

    def splotrc
      ".splotrc"
    end
  end
end
